/**
 * Copyright (c) 2025 Enzien Audio, Ltd.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions, and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the phrase "powered by heavy",
 *    the heavy logo, and a hyperlink to https://enzienaudio.com, all in a visible
 *    form.
 * 
 *   2.1 If the Application is distributed in a store system (for example,
 *       the Apple "App Store" or "Google Play"), the phrase "powered by heavy"
 *       shall be included in the app description or the copyright text as well as
 *       the in the app itself. The heavy logo will shall be visible in the app
 *       itself as well.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#include "Heavy_WSTD_FLDR.hpp"

#include <new>

#define Context(_c) static_cast<Heavy_WSTD_FLDR *>(_c)


/*
 * C Functions
 */

extern "C" {
  HV_EXPORT HeavyContextInterface *hv_WSTD_FLDR_new(double sampleRate) {
    // allocate aligned memory
    void *ptr = hv_malloc(sizeof(Heavy_WSTD_FLDR));
    // ensure non-null
    if (!ptr) return nullptr;
    // call constructor
    new(ptr) Heavy_WSTD_FLDR(sampleRate);
    return Context(ptr);
  }

  HV_EXPORT HeavyContextInterface *hv_WSTD_FLDR_new_with_options(double sampleRate,
      int poolKb, int inQueueKb, int outQueueKb) {
    // allocate aligned memory
    void *ptr = hv_malloc(sizeof(Heavy_WSTD_FLDR));
    // ensure non-null
    if (!ptr) return nullptr;
    // call constructor
    new(ptr) Heavy_WSTD_FLDR(sampleRate, poolKb, inQueueKb, outQueueKb);
    return Context(ptr);
  }

  HV_EXPORT void hv_WSTD_FLDR_free(HeavyContextInterface *instance) {
    // call destructor
    Context(instance)->~Heavy_WSTD_FLDR();
    // free memory
    hv_free(instance);
  }
} // extern "C"







/*
 * Class Functions
 */

Heavy_WSTD_FLDR::Heavy_WSTD_FLDR(double sampleRate, int poolKb, int inQueueKb, int outQueueKb)
    : HeavyContext(sampleRate, poolKb, inQueueKb, outQueueKb) {
  numBytes += sLine_init(&sLine_4mtXeDd3);
  numBytes += sEnv_init(&sEnv_qdUVNvVK, 512, 256);
  numBytes += sEnv_init(&sEnv_n1iQiX09, 512, 256);
  numBytes += sEnv_init(&sEnv_3yBGsFvL, 512, 256);
  numBytes += sEnv_init(&sEnv_4j0v9jVS, 512, 256);
  numBytes += sLine_init(&sLine_Py16NFVW);
  numBytes += sLine_init(&sLine_SXKkNsV4);
  numBytes += cVar_init_f(&cVar_OXe6zkEU, 0.0f);
  numBytes += cVar_init_f(&cVar_A7H1C2I0, 0.0f);
  numBytes += sVarf_init(&sVarf_S6dVnCsj, 0.0f, 0.0f, false);
  numBytes += cVar_init_f(&cVar_4BT0EhSw, 0.0f);
  numBytes += cIf_init(&cIf_GUZxd4I8, false);
  numBytes += cIf_init(&cIf_xBntjQ0N, false);
  numBytes += cBinop_init(&cBinop_rjX2mdES, 0.0f); // __sub
  numBytes += sVarf_init(&sVarf_zCdL60Ya, 0.0f, 0.0f, false);
  numBytes += cVar_init_f(&cVar_5e2sKyDz, 0.0f);
  numBytes += cIf_init(&cIf_SC0qUaaC, false);
  numBytes += cIf_init(&cIf_jCSbWVIL, false);
  numBytes += cBinop_init(&cBinop_4gE23Gni, 0.0f); // __sub
  numBytes += cVar_init_f(&cVar_rL2626FB, 1.0f);
  numBytes += sVarf_init(&sVarf_tYDAqoPn, 0.0f, 0.0f, false);
  numBytes += sVarf_init(&sVarf_FWGGvTg0, 0.0f, 0.0f, false);
  numBytes += sVarf_init(&sVarf_kl16ValL, 0.0f, 0.0f, false);
  numBytes += sVarf_init(&sVarf_Go5uBuiR, 0.0f, 0.0f, false);
  
  // schedule a message to trigger all loadbangs via the __hv_init receiver
  scheduleMessageForReceiver(0xCE5CC65B, msg_initWithBang(HV_MESSAGE_ON_STACK(1), 0));
}

Heavy_WSTD_FLDR::~Heavy_WSTD_FLDR() {
  sEnv_free(&sEnv_qdUVNvVK);
  sEnv_free(&sEnv_n1iQiX09);
  sEnv_free(&sEnv_3yBGsFvL);
  sEnv_free(&sEnv_4j0v9jVS);
}

HvTable *Heavy_WSTD_FLDR::getTableForHash(hv_uint32_t tableHash) {
  return nullptr;
}

void Heavy_WSTD_FLDR::scheduleMessageForReceiver(hv_uint32_t receiverHash, HvMessage *m) {
  switch (receiverHash) {
    case 0xE42C892E: { // Fldr
      mq_addMessageByTimestamp(&mq, m, 0, &cReceive_s8eMEhs6_sendMessage);
      break;
    }
    case 0xCB23DC60: { // Lmtr
      mq_addMessageByTimestamp(&mq, m, 0, &cReceive_1FkrRRpj_sendMessage);
      break;
    }
    case 0x97D46AC6: { // Mix
      mq_addMessageByTimestamp(&mq, m, 0, &cReceive_etJ94GQB_sendMessage);
      break;
    }
    case 0xCF8296B9: { // 1001-mix
      mq_addMessageByTimestamp(&mq, m, 0, &cReceive_XuC6LsO4_sendMessage);
      break;
    }
    case 0xCE5CC65B: { // __hv_init
      mq_addMessageByTimestamp(&mq, m, 0, &cReceive_FSSyGdQu_sendMessage);
      break;
    }
    default: return;
  }
}

int Heavy_WSTD_FLDR::getParameterInfo(int index, HvParameterInfo *info) {
  if (info != nullptr) {
    switch (index) {
      case 0: {
        info->name = "Fldr";
        info->hash = 0xE42C892E;
        info->type = HvParameterType::HV_PARAM_TYPE_PARAMETER_IN;
        info->minVal = 1.0f;
        info->maxVal = 13.37f;
        info->defaultVal = 1.0f;
        break;
      }
      case 1: {
        info->name = "Lmtr";
        info->hash = 0xCB23DC60;
        info->type = HvParameterType::HV_PARAM_TYPE_PARAMETER_IN;
        info->minVal = 0.0f;
        info->maxVal = 1.0f;
        info->defaultVal = 1.0f;
        break;
      }
      case 2: {
        info->name = "Mix";
        info->hash = 0x97D46AC6;
        info->type = HvParameterType::HV_PARAM_TYPE_PARAMETER_IN;
        info->minVal = 0.0f;
        info->maxVal = 100.0f;
        info->defaultVal = 50.0f;
        break;
      }
      default: {
        info->name = "invalid parameter index";
        info->hash = 0;
        info->type = HvParameterType::HV_PARAM_TYPE_PARAMETER_IN;
        info->minVal = 0.0f;
        info->maxVal = 0.0f;
        info->defaultVal = 0.0f;
        break;
      }
    }
  }
  return 3;
}



/*
 * Send Function Implementations
 */


void Heavy_WSTD_FLDR::cBinop_Blx9xkxy_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_k_onMessage(_c, NULL, HV_BINOP_MAX, 0.0f, 0, m, &cBinop_CaHXMadz_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_CaHXMadz_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_k_onMessage(_c, NULL, HV_BINOP_MULTIPLY, 2.0f, 0, m, &cBinop_ndu9I1xW_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_ndu9I1xW_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_k_onMessage(_c, NULL, HV_BINOP_SUBTRACT, 1.0f, 0, m, &cBinop_J0OzpbB4_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_J0OzpbB4_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cCast_onMessage(_c, HV_CAST_FLOAT, 0, m, &cCast_ZleqAwGu_sendMessage);
  cCast_onMessage(_c, HV_CAST_FLOAT, 0, m, &cCast_M3di3MYp_sendMessage);
}

void Heavy_WSTD_FLDR::cCast_M3di3MYp_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cMsg_E4HNf8Td_sendMessage(_c, 0, m);
}

void Heavy_WSTD_FLDR::cCast_ZleqAwGu_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_k_onMessage(_c, NULL, HV_BINOP_ADD, 1.0f, 0, m, &cBinop_zJbEUjKd_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_zJbEUjKd_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_k_onMessage(_c, NULL, HV_BINOP_MULTIPLY, 0.5f, 0, m, &cBinop_A8b4Ju0G_sendMessage);
}

void Heavy_WSTD_FLDR::cMsg_E4HNf8Td_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *const n) {
  HvMessage *m = nullptr;
  m = HV_MESSAGE_ON_STACK(2);
  msg_init(m, 2, msg_getTimestamp(n));
  msg_setFloat(m, 0, 1.0f);
  msg_setElementToFrom(m, 1, n, 0);
  cBinop_k_onMessage(_c, NULL, HV_BINOP_SUBTRACT, 0.0f, 0, m, &cBinop_JPH2yCq9_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_JPH2yCq9_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_k_onMessage(_c, NULL, HV_BINOP_MULTIPLY, 0.5f, 0, m, &cBinop_a3RVo6aL_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_A8b4Ju0G_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cMsg_Gf9bTgbH_sendMessage(_c, 0, m);
}

void Heavy_WSTD_FLDR::cBinop_a3RVo6aL_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cMsg_URRZG7c1_sendMessage(_c, 0, m);
}

void Heavy_WSTD_FLDR::cVar_OXe6zkEU_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cSend_ddRf5nLe_sendMessage(_c, 0, m);
}

void Heavy_WSTD_FLDR::cCast_hjlNAKVS_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cMsg_xk53R03m_sendMessage(_c, 0, m);
  cMsg_TuOXAjLL_sendMessage(_c, 0, m);
}

void Heavy_WSTD_FLDR::cVar_A7H1C2I0_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cMsg_sLVsFFYY_sendMessage(_c, 0, m);
}

void Heavy_WSTD_FLDR::cVar_4BT0EhSw_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_onMessage(_c, &Context(_c)->cBinop_rjX2mdES, HV_BINOP_SUBTRACT, 0, m, &cBinop_rjX2mdES_sendMessage);
}

void Heavy_WSTD_FLDR::cIf_GUZxd4I8_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  switch (letIn) {
    case 0: {
      break;
    }
    case 1: {
      sVarf_onMessage(_c, &Context(_c)->sVarf_S6dVnCsj, m);
      break;
    }
    default: return;
  }
}

void Heavy_WSTD_FLDR::cIf_xBntjQ0N_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  switch (letIn) {
    case 0: {
      break;
    }
    case 1: {
      sVarf_onMessage(_c, &Context(_c)->sVarf_S6dVnCsj, m);
      break;
    }
    default: return;
  }
}

void Heavy_WSTD_FLDR::sEnv_qdUVNvVK_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cVar_onMessage(_c, &Context(_c)->cVar_4BT0EhSw, 0, m, &cVar_4BT0EhSw_sendMessage);
}

void Heavy_WSTD_FLDR::sEnv_n1iQiX09_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cCast_onMessage(_c, HV_CAST_FLOAT, 0, m, &cCast_vdgZvXPb_sendMessage);
  cCast_onMessage(_c, HV_CAST_FLOAT, 0, m, &cCast_AukbqFmE_sendMessage);
  cCast_onMessage(_c, HV_CAST_BANG, 0, m, &cCast_sCk4zqJ8_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_rjX2mdES_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cMsg_QR0gfPPd_sendMessage(_c, 0, m);
}

void Heavy_WSTD_FLDR::cMsg_QR0gfPPd_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *const n) {
  HvMessage *m = nullptr;
  m = HV_MESSAGE_ON_STACK(2);
  msg_init(m, 2, msg_getTimestamp(n));
  msg_setFloat(m, 0, 100.0f);
  msg_setElementToFrom(m, 1, n, 0);
  cBinop_k_onMessage(_c, NULL, HV_BINOP_SUBTRACT, 0.0f, 0, m, &cBinop_zC2pwnjX_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_zC2pwnjX_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_k_onMessage(_c, NULL, HV_BINOP_MIN, 100.0f, 0, m, &cBinop_EtPBzWWD_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_YMuzQT3t_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cCast_onMessage(_c, HV_CAST_FLOAT, 0, m, &cCast_BlRRwx8j_sendMessage);
  cCast_onMessage(_c, HV_CAST_BANG, 0, m, &cCast_fh6dhfr1_sendMessage);
}

void Heavy_WSTD_FLDR::cCast_BlRRwx8j_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cIf_onMessage(_c, &Context(_c)->cIf_GUZxd4I8, 1, m, &cIf_GUZxd4I8_sendMessage);
}

void Heavy_WSTD_FLDR::cCast_fh6dhfr1_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cMsg_mp2ZFHWQ_sendMessage(_c, 0, m);
}

void Heavy_WSTD_FLDR::cBinop_ryDYU6oF_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cIf_onMessage(_c, &Context(_c)->cIf_xBntjQ0N, 1, m, &cIf_xBntjQ0N_sendMessage);
}

void Heavy_WSTD_FLDR::cMsg_mp2ZFHWQ_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *const n) {
  HvMessage *m = nullptr;
  m = HV_MESSAGE_ON_STACK(1);
  msg_init(m, 1, msg_getTimestamp(n));
  msg_setFloat(m, 0, 100.0f);
  cIf_onMessage(_c, &Context(_c)->cIf_GUZxd4I8, 0, m, &cIf_GUZxd4I8_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_EtPBzWWD_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cIf_onMessage(_c, &Context(_c)->cIf_xBntjQ0N, 0, m, &cIf_xBntjQ0N_sendMessage);
}

void Heavy_WSTD_FLDR::cCast_vdgZvXPb_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
}

void Heavy_WSTD_FLDR::cCast_sCk4zqJ8_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cVar_onMessage(_c, &Context(_c)->cVar_4BT0EhSw, 0, m, &cVar_4BT0EhSw_sendMessage);
}

void Heavy_WSTD_FLDR::cCast_AukbqFmE_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_onMessage(_c, &Context(_c)->cBinop_rjX2mdES, HV_BINOP_SUBTRACT, 1, m, &cBinop_rjX2mdES_sendMessage);
}

void Heavy_WSTD_FLDR::cVar_5e2sKyDz_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_onMessage(_c, &Context(_c)->cBinop_4gE23Gni, HV_BINOP_SUBTRACT, 0, m, &cBinop_4gE23Gni_sendMessage);
}

void Heavy_WSTD_FLDR::cIf_SC0qUaaC_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  switch (letIn) {
    case 0: {
      break;
    }
    case 1: {
      sVarf_onMessage(_c, &Context(_c)->sVarf_zCdL60Ya, m);
      break;
    }
    default: return;
  }
}

void Heavy_WSTD_FLDR::cIf_jCSbWVIL_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  switch (letIn) {
    case 0: {
      break;
    }
    case 1: {
      sVarf_onMessage(_c, &Context(_c)->sVarf_zCdL60Ya, m);
      break;
    }
    default: return;
  }
}

void Heavy_WSTD_FLDR::sEnv_3yBGsFvL_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cVar_onMessage(_c, &Context(_c)->cVar_5e2sKyDz, 0, m, &cVar_5e2sKyDz_sendMessage);
}

void Heavy_WSTD_FLDR::sEnv_4j0v9jVS_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cCast_onMessage(_c, HV_CAST_FLOAT, 0, m, &cCast_H2rL1sKe_sendMessage);
  cCast_onMessage(_c, HV_CAST_FLOAT, 0, m, &cCast_nW5Y5uuG_sendMessage);
  cCast_onMessage(_c, HV_CAST_BANG, 0, m, &cCast_YkS0jmqa_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_4gE23Gni_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cMsg_OG5SLVl9_sendMessage(_c, 0, m);
}

void Heavy_WSTD_FLDR::cMsg_OG5SLVl9_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *const n) {
  HvMessage *m = nullptr;
  m = HV_MESSAGE_ON_STACK(2);
  msg_init(m, 2, msg_getTimestamp(n));
  msg_setFloat(m, 0, 100.0f);
  msg_setElementToFrom(m, 1, n, 0);
  cBinop_k_onMessage(_c, NULL, HV_BINOP_SUBTRACT, 0.0f, 0, m, &cBinop_PvUJpD8B_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_PvUJpD8B_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_k_onMessage(_c, NULL, HV_BINOP_MIN, 100.0f, 0, m, &cBinop_BZHxabY7_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_o00Cw8ZM_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cCast_onMessage(_c, HV_CAST_FLOAT, 0, m, &cCast_1yK8J2Xi_sendMessage);
  cCast_onMessage(_c, HV_CAST_BANG, 0, m, &cCast_5acCGgWD_sendMessage);
}

void Heavy_WSTD_FLDR::cCast_5acCGgWD_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cMsg_Uf12MfzV_sendMessage(_c, 0, m);
}

void Heavy_WSTD_FLDR::cCast_1yK8J2Xi_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cIf_onMessage(_c, &Context(_c)->cIf_SC0qUaaC, 1, m, &cIf_SC0qUaaC_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_vhHphpFb_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cIf_onMessage(_c, &Context(_c)->cIf_jCSbWVIL, 1, m, &cIf_jCSbWVIL_sendMessage);
}

void Heavy_WSTD_FLDR::cMsg_Uf12MfzV_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *const n) {
  HvMessage *m = nullptr;
  m = HV_MESSAGE_ON_STACK(1);
  msg_init(m, 1, msg_getTimestamp(n));
  msg_setFloat(m, 0, 100.0f);
  cIf_onMessage(_c, &Context(_c)->cIf_SC0qUaaC, 0, m, &cIf_SC0qUaaC_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_BZHxabY7_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cIf_onMessage(_c, &Context(_c)->cIf_jCSbWVIL, 0, m, &cIf_jCSbWVIL_sendMessage);
}

void Heavy_WSTD_FLDR::cCast_nW5Y5uuG_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_onMessage(_c, &Context(_c)->cBinop_4gE23Gni, HV_BINOP_SUBTRACT, 1, m, &cBinop_4gE23Gni_sendMessage);
}

void Heavy_WSTD_FLDR::cCast_H2rL1sKe_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
}

void Heavy_WSTD_FLDR::cCast_YkS0jmqa_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cVar_onMessage(_c, &Context(_c)->cVar_5e2sKyDz, 0, m, &cVar_5e2sKyDz_sendMessage);
}

void Heavy_WSTD_FLDR::cVar_rL2626FB_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_k_onMessage(_c, NULL, HV_BINOP_EQ, 0.0f, 0, m, &cBinop_nCpPmiEw_sendMessage);
  cBinop_k_onMessage(_c, NULL, HV_BINOP_EQ, 0.0f, 0, m, &cBinop_YMuzQT3t_sendMessage);
  cBinop_k_onMessage(_c, NULL, HV_BINOP_EQ, 1.0f, 0, m, &cBinop_ryDYU6oF_sendMessage);
  cBinop_k_onMessage(_c, NULL, HV_BINOP_EQ, 0.0f, 0, m, &cBinop_o00Cw8ZM_sendMessage);
  cBinop_k_onMessage(_c, NULL, HV_BINOP_EQ, 1.0f, 0, m, &cBinop_vhHphpFb_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_nCpPmiEw_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cVar_onMessage(_c, &Context(_c)->cVar_rL2626FB, 1, m, &cVar_rL2626FB_sendMessage);
}

void Heavy_WSTD_FLDR::cMsg_Gf9bTgbH_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *const n) {
  HvMessage *m = nullptr;
  m = HV_MESSAGE_ON_STACK(2);
  msg_init(m, 2, msg_getTimestamp(n));
  msg_setElementToFrom(m, 0, n, 0);
  msg_setFloat(m, 1, 100.0f);
  sLine_onMessage(_c, &Context(_c)->sLine_SXKkNsV4, 0, m, NULL);
}

void Heavy_WSTD_FLDR::cMsg_URRZG7c1_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *const n) {
  HvMessage *m = nullptr;
  m = HV_MESSAGE_ON_STACK(2);
  msg_init(m, 2, msg_getTimestamp(n));
  msg_setElementToFrom(m, 0, n, 0);
  msg_setFloat(m, 1, 100.0f);
  sLine_onMessage(_c, &Context(_c)->sLine_Py16NFVW, 0, m, NULL);
}

void Heavy_WSTD_FLDR::cSend_ddRf5nLe_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cReceive_XuC6LsO4_sendMessage(_c, 0, m);
}

void Heavy_WSTD_FLDR::cBinop_524Ttv0r_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cVar_onMessage(_c, &Context(_c)->cVar_OXe6zkEU, 0, m, &cVar_OXe6zkEU_sendMessage);
}

void Heavy_WSTD_FLDR::cMsg_TuOXAjLL_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *const n) {
  HvMessage *m = nullptr;
  m = HV_MESSAGE_ON_STACK(1);
  msg_init(m, 1, msg_getTimestamp(n));
  msg_setFloat(m, 0, 50.0f);
  cBinop_k_onMessage(_c, NULL, HV_BINOP_DIVIDE, 100.0f, 0, m, &cBinop_524Ttv0r_sendMessage);
}

void Heavy_WSTD_FLDR::cMsg_sLVsFFYY_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *const n) {
  HvMessage *m = nullptr;
  m = HV_MESSAGE_ON_STACK(2);
  msg_init(m, 2, msg_getTimestamp(n));
  msg_setElementToFrom(m, 0, n, 0);
  msg_setFloat(m, 1, 20.0f);
  sLine_onMessage(_c, &Context(_c)->sLine_4mtXeDd3, 0, m, NULL);
}

void Heavy_WSTD_FLDR::cMsg_xk53R03m_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *const n) {
  HvMessage *m = nullptr;
  m = HV_MESSAGE_ON_STACK(1);
  msg_init(m, 1, msg_getTimestamp(n));
  msg_setFloat(m, 0, 1.0f);
  cVar_onMessage(_c, &Context(_c)->cVar_A7H1C2I0, 0, m, &cVar_A7H1C2I0_sendMessage);
  cVar_onMessage(_c, &Context(_c)->cVar_rL2626FB, 0, m, &cVar_rL2626FB_sendMessage);
}

void Heavy_WSTD_FLDR::cReceive_XuC6LsO4_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_k_onMessage(_c, NULL, HV_BINOP_MIN, 1.0f, 0, m, &cBinop_Blx9xkxy_sendMessage);
}

void Heavy_WSTD_FLDR::cReceive_etJ94GQB_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_k_onMessage(_c, NULL, HV_BINOP_DIVIDE, 100.0f, 0, m, &cBinop_524Ttv0r_sendMessage);
}

void Heavy_WSTD_FLDR::cReceive_FSSyGdQu_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cCast_onMessage(_c, HV_CAST_BANG, 0, m, &cCast_hjlNAKVS_sendMessage);
}

void Heavy_WSTD_FLDR::cReceive_1FkrRRpj_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cVar_onMessage(_c, &Context(_c)->cVar_rL2626FB, 0, m, &cVar_rL2626FB_sendMessage);
}

void Heavy_WSTD_FLDR::cReceive_s8eMEhs6_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cVar_onMessage(_c, &Context(_c)->cVar_A7H1C2I0, 0, m, &cVar_A7H1C2I0_sendMessage);
}




/*
 * Context Process Implementation
 */

int Heavy_WSTD_FLDR::process(float **inputBuffers, float **outputBuffers, int n) {
  while (hLp_hasData(&inQueue)) {
    hv_uint32_t numBytes = 0;
    ReceiverMessagePair *p = reinterpret_cast<ReceiverMessagePair *>(hLp_getReadBuffer(&inQueue, &numBytes));
    hv_assert(numBytes >= sizeof(ReceiverMessagePair));
    scheduleMessageForReceiver(p->receiverHash, &p->msg);
    hLp_consume(&inQueue);
  }

  sendBangToReceiver(0xDD21C0EB); // send to __hv_bang~ on next cycle
  const int n4 = n & ~HV_N_SIMD_MASK; // ensure that the block size is a multiple of HV_N_SIMD

  // temporary signal vars
  hv_bufferf_t Bf0, Bf1, Bf2, Bf3;

  // input and output vars
  hv_bufferf_t O0, O1;
  hv_bufferf_t I0, I1;

  // declare and init the zero buffer
  hv_bufferf_t ZERO; __hv_zero_f(VOf(ZERO));

  hv_uint32_t nextBlock = blockStartTimestamp;
  for (int n = 0; n < n4; n += HV_N_SIMD) {

    // process all of the messages for this block
    nextBlock += HV_N_SIMD;
    while (mq_hasMessageBefore(&mq, nextBlock)) {
      MessageNode *const node = mq_peek(&mq);
      node->sendMessage(this, node->let, node->m);
      mq_pop(&mq);
    }

    // load input buffers
    __hv_load_f(inputBuffers[0]+n, VOf(I0));
    __hv_load_f(inputBuffers[1]+n, VOf(I1));

    // zero output buffers
    __hv_zero_f(VOf(O0));
    __hv_zero_f(VOf(O1));

    // process all signal functions
    __hv_varwrite_f(&sVarf_tYDAqoPn, VIf(I0));
    __hv_line_f(&sLine_4mtXeDd3, VOf(Bf0));
    __hv_var_k_f(VOf(Bf1), 1.0f, 1.0f, 1.0f, 1.0f, 1.0f, 1.0f, 1.0f, 1.0f);
    __hv_fma_f(VIf(I0), VIf(Bf0), VIf(Bf1), VOf(Bf1));
    __hv_var_k_f(VOf(Bf2), 4.0f, 4.0f, 4.0f, 4.0f, 4.0f, 4.0f, 4.0f, 4.0f);
    __hv_div_f(VIf(Bf1), VIf(Bf2), VOf(Bf2));
    __hv_floor_f(VIf(Bf2), VOf(Bf1));
    __hv_sub_f(VIf(Bf2), VIf(Bf1), VOf(Bf1));
    __hv_var_k_f(VOf(Bf2), 0.5f, 0.5f, 0.5f, 0.5f, 0.5f, 0.5f, 0.5f, 0.5f);
    __hv_sub_f(VIf(Bf1), VIf(Bf2), VOf(Bf2));
    __hv_abs_f(VIf(Bf2), VOf(Bf2));
    __hv_var_k_f(VOf(Bf1), 4.0f, 4.0f, 4.0f, 4.0f, 4.0f, 4.0f, 4.0f, 4.0f);
    __hv_var_k_f(VOf(Bf3), 1.0f, 1.0f, 1.0f, 1.0f, 1.0f, 1.0f, 1.0f, 1.0f);
    __hv_fms_f(VIf(Bf2), VIf(Bf1), VIf(Bf3), VOf(Bf3));
    __hv_neg_f(VIf(Bf3), VOf(Bf3));
    sEnv_process(this, &sEnv_qdUVNvVK, VIf(Bf3), &sEnv_qdUVNvVK_sendMessage);
    sEnv_process(this, &sEnv_n1iQiX09, VIf(I0), &sEnv_n1iQiX09_sendMessage);
    __hv_varread_f(&sVarf_S6dVnCsj, VOf(Bf1));
    __hv_var_k_f(VOf(Bf2), 100.0f, 100.0f, 100.0f, 100.0f, 100.0f, 100.0f, 100.0f, 100.0f);
    __hv_sub_f(VIf(Bf1), VIf(Bf2), VOf(Bf2));
    __hv_var_k_f(VOf(Bf1), 0.115129254649702f, 0.115129254649702f, 0.115129254649702f, 0.115129254649702f, 0.115129254649702f, 0.115129254649702f, 0.115129254649702f, 0.115129254649702f);
    __hv_mul_f(VIf(Bf2), VIf(Bf1), VOf(Bf1));
    __hv_exp_f(VIf(Bf1), VOf(Bf1));
    __hv_mul_f(VIf(Bf3), VIf(Bf1), VOf(Bf1));
    __hv_varwrite_f(&sVarf_FWGGvTg0, VIf(Bf1));
    __hv_varwrite_f(&sVarf_kl16ValL, VIf(I1));
    __hv_var_k_f(VOf(Bf1), 1.0f, 1.0f, 1.0f, 1.0f, 1.0f, 1.0f, 1.0f, 1.0f);
    __hv_fma_f(VIf(I1), VIf(Bf0), VIf(Bf1), VOf(Bf1));
    __hv_var_k_f(VOf(Bf0), 4.0f, 4.0f, 4.0f, 4.0f, 4.0f, 4.0f, 4.0f, 4.0f);
    __hv_div_f(VIf(Bf1), VIf(Bf0), VOf(Bf0));
    __hv_floor_f(VIf(Bf0), VOf(Bf1));
    __hv_sub_f(VIf(Bf0), VIf(Bf1), VOf(Bf1));
    __hv_var_k_f(VOf(Bf0), 0.5f, 0.5f, 0.5f, 0.5f, 0.5f, 0.5f, 0.5f, 0.5f);
    __hv_sub_f(VIf(Bf1), VIf(Bf0), VOf(Bf0));
    __hv_abs_f(VIf(Bf0), VOf(Bf0));
    __hv_var_k_f(VOf(Bf1), 4.0f, 4.0f, 4.0f, 4.0f, 4.0f, 4.0f, 4.0f, 4.0f);
    __hv_var_k_f(VOf(Bf3), 1.0f, 1.0f, 1.0f, 1.0f, 1.0f, 1.0f, 1.0f, 1.0f);
    __hv_fms_f(VIf(Bf0), VIf(Bf1), VIf(Bf3), VOf(Bf3));
    __hv_neg_f(VIf(Bf3), VOf(Bf3));
    sEnv_process(this, &sEnv_3yBGsFvL, VIf(Bf3), &sEnv_3yBGsFvL_sendMessage);
    sEnv_process(this, &sEnv_4j0v9jVS, VIf(I1), &sEnv_4j0v9jVS_sendMessage);
    __hv_varread_f(&sVarf_zCdL60Ya, VOf(Bf1));
    __hv_var_k_f(VOf(Bf0), 100.0f, 100.0f, 100.0f, 100.0f, 100.0f, 100.0f, 100.0f, 100.0f);
    __hv_sub_f(VIf(Bf1), VIf(Bf0), VOf(Bf0));
    __hv_var_k_f(VOf(Bf1), 0.115129254649702f, 0.115129254649702f, 0.115129254649702f, 0.115129254649702f, 0.115129254649702f, 0.115129254649702f, 0.115129254649702f, 0.115129254649702f);
    __hv_mul_f(VIf(Bf0), VIf(Bf1), VOf(Bf1));
    __hv_exp_f(VIf(Bf1), VOf(Bf1));
    __hv_mul_f(VIf(Bf3), VIf(Bf1), VOf(Bf1));
    __hv_varwrite_f(&sVarf_Go5uBuiR, VIf(Bf1));
    __hv_varread_f(&sVarf_tYDAqoPn, VOf(Bf1));
    __hv_line_f(&sLine_Py16NFVW, VOf(Bf3));
    __hv_varread_f(&sVarf_FWGGvTg0, VOf(Bf0));
    __hv_line_f(&sLine_SXKkNsV4, VOf(Bf2));
    __hv_mul_f(VIf(Bf0), VIf(Bf2), VOf(Bf0));
    __hv_fma_f(VIf(Bf1), VIf(Bf3), VIf(Bf0), VOf(Bf0));
    __hv_add_f(VIf(Bf0), VIf(O0), VOf(O0));
    __hv_varread_f(&sVarf_kl16ValL, VOf(Bf0));
    __hv_varread_f(&sVarf_Go5uBuiR, VOf(Bf1));
    __hv_mul_f(VIf(Bf1), VIf(Bf2), VOf(Bf2));
    __hv_fma_f(VIf(Bf0), VIf(Bf3), VIf(Bf2), VOf(Bf2));
    __hv_add_f(VIf(Bf2), VIf(O1), VOf(O1));

    // save output vars to output buffer
    __hv_store_f(outputBuffers[0]+n, VIf(O0));
    __hv_store_f(outputBuffers[1]+n, VIf(O1));
  }

  blockStartTimestamp = nextBlock;

  return n4; // return the number of frames processed

}

int Heavy_WSTD_FLDR::processInline(float *inputBuffers, float *outputBuffers, int n4) {
  hv_assert(!(n4 & HV_N_SIMD_MASK)); // ensure that n4 is a multiple of HV_N_SIMD

  // define the heavy input buffer for 2 channel(s)
  float **const bIn = reinterpret_cast<float **>(hv_alloca(2*sizeof(float *)));
  bIn[0] = inputBuffers+(0*n4);
  bIn[1] = inputBuffers+(1*n4);

  // define the heavy output buffer for 2 channel(s)
  float **const bOut = reinterpret_cast<float **>(hv_alloca(2*sizeof(float *)));
  bOut[0] = outputBuffers+(0*n4);
  bOut[1] = outputBuffers+(1*n4);

  int n = process(bIn, bOut, n4);
  return n;
}

int Heavy_WSTD_FLDR::processInlineInterleaved(float *inputBuffers, float *outputBuffers, int n4) {
  hv_assert(n4 & ~HV_N_SIMD_MASK); // ensure that n4 is a multiple of HV_N_SIMD

  // define the heavy input buffer for 2 channel(s), uninterleave
  float *const bIn = reinterpret_cast<float *>(hv_alloca(2*n4*sizeof(float)));
  #if HV_SIMD_SSE || HV_SIMD_AVX
  for (int i = 0, j = 0; j < n4; j += 4, i += 8) {
    __m128 a = _mm_load_ps(inputBuffers+i);                // LRLR
    __m128 b = _mm_load_ps(inputBuffers+4+i);              // LRLR
    __m128 x = _mm_shuffle_ps(a, b, _MM_SHUFFLE(2,0,2,0)); // LLLL
    __m128 y = _mm_shuffle_ps(a, b, _MM_SHUFFLE(3,1,3,1)); // RRRR
    _mm_store_ps(bIn+j, x);
    _mm_store_ps(bIn+n4+j, y);
  }
  #elif HV_SIMD_NEON
  for (int i = 0, j = 0; j < n4; j += 4, i += 8) {
    float32x4x2_t a = vld2q_f32(inputBuffers+i); // load and uninterleave
    vst1q_f32(bIn+j, a.val[0]);
    vst1q_f32(bIn+n4+j, a.val[1]);
  }
  #else // HV_SIMD_NONE
  for (int j = 0; j < n4; ++j) {
    bIn[0*n4+j] = inputBuffers[0+2*j];
    bIn[1*n4+j] = inputBuffers[1+2*j];
  }
  #endif

  // define the heavy output buffer for 2 channel(s)
  float *const bOut = reinterpret_cast<float *>(hv_alloca(2*n4*sizeof(float)));

  int n = processInline(bIn, bOut, n4);

  // interleave the heavy output into the output buffer
  #if HV_SIMD_AVX
  for (int i = 0, j = 0; j < n4; j += 8, i += 16) {
    __m256 x = _mm256_load_ps(bOut+j);    // LLLLLLLL
    __m256 y = _mm256_load_ps(bOut+n4+j); // RRRRRRRR
    __m256 a = _mm256_unpacklo_ps(x, y);  // LRLRLRLR
    __m256 b = _mm256_unpackhi_ps(x, y);  // LRLRLRLR
    _mm256_store_ps(outputBuffers+i, a);
    _mm256_store_ps(outputBuffers+8+i, b);
  }
  #elif HV_SIMD_SSE
  for (int i = 0, j = 0; j < n4; j += 4, i += 8) {
    __m128 x = _mm_load_ps(bOut+j);    // LLLL
    __m128 y = _mm_load_ps(bOut+n4+j); // RRRR
    __m128 a = _mm_unpacklo_ps(x, y);  // LRLR
    __m128 b = _mm_unpackhi_ps(x, y);  // LRLR
    _mm_store_ps(outputBuffers+i, a);
    _mm_store_ps(outputBuffers+4+i, b);
  }
  #elif HV_SIMD_NEON
  // https://community.arm.com/groups/processors/blog/2012/03/13/coding-for-neon--part-5-rearranging-vectors
  for (int i = 0, j = 0; j < n4; j += 4, i += 8) {
    float32x4_t x = vld1q_f32(bOut+j);
    float32x4_t y = vld1q_f32(bOut+n4+j);
    float32x4x2_t z = {x, y};
    vst2q_f32(outputBuffers+i, z); // interleave and store
  }
  #else // HV_SIMD_NONE
  for (int i = 0; i < 2; ++i) {
    for (int j = 0; j < n4; ++j) {
      outputBuffers[i+2*j] = bOut[i*n4+j];
    }
  }
  #endif

  return n;
}
