/**
 * Copyright (c) 2025 Enzien Audio, Ltd.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions, and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the phrase "powered by heavy",
 *    the heavy logo, and a hyperlink to https://enzienaudio.com, all in a visible
 *    form.
 * 
 *   2.1 If the Application is distributed in a store system (for example,
 *       the Apple "App Store" or "Google Play"), the phrase "powered by heavy"
 *       shall be included in the app description or the copyright text as well as
 *       the in the app itself. The heavy logo will shall be visible in the app
 *       itself as well.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#ifndef _HEAVY_CONTEXT_WSTD_FLDR_HPP_
#define _HEAVY_CONTEXT_WSTD_FLDR_HPP_

// object includes
#include "HeavyContext.hpp"
#include "HvControlVar.h"
#include "HvControlCast.h"
#include "HvSignalEnvelope.h"
#include "HvMath.h"
#include "HvSignalVar.h"
#include "HvControlIf.h"
#include "HvSignalLine.h"
#include "HvControlBinop.h"

class Heavy_WSTD_FLDR : public HeavyContext {

 public:
  Heavy_WSTD_FLDR(double sampleRate, int poolKb=10, int inQueueKb=2, int outQueueKb=0);
  ~Heavy_WSTD_FLDR();

  const char *getName() override { return "WSTD_FLDR"; }
  int getNumInputChannels() override { return 2; }
  int getNumOutputChannels() override { return 2; }

  int process(float **inputBuffers, float **outputBuffer, int n) override;
  int processInline(float *inputBuffers, float *outputBuffer, int n) override;
  int processInlineInterleaved(float *inputBuffers, float *outputBuffer, int n) override;

  int getParameterInfo(int index, HvParameterInfo *info) override;
  struct Parameter {
    struct In {
      enum ParameterIn : hv_uint32_t {
        FLDR = 0xE42C892E, // Fldr
        LMTR = 0xCB23DC60, // Lmtr
        MIX = 0x97D46AC6, // Mix
      };
    };
  };

 private:
  HvTable *getTableForHash(hv_uint32_t tableHash) override;
  void scheduleMessageForReceiver(hv_uint32_t receiverHash, HvMessage *m) override;

  // static sendMessage functions
  static void cBinop_Blx9xkxy_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_CaHXMadz_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_ndu9I1xW_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_J0OzpbB4_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_M3di3MYp_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_ZleqAwGu_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_zJbEUjKd_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_E4HNf8Td_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_JPH2yCq9_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_A8b4Ju0G_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_a3RVo6aL_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_OXe6zkEU_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_hjlNAKVS_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_A7H1C2I0_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_4BT0EhSw_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_GUZxd4I8_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_xBntjQ0N_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_qdUVNvVK_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_n1iQiX09_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_rjX2mdES_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_QR0gfPPd_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_zC2pwnjX_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_YMuzQT3t_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_BlRRwx8j_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_fh6dhfr1_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_ryDYU6oF_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_mp2ZFHWQ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_EtPBzWWD_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_vdgZvXPb_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_sCk4zqJ8_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_AukbqFmE_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_5e2sKyDz_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_SC0qUaaC_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_jCSbWVIL_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_3yBGsFvL_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_4j0v9jVS_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_4gE23Gni_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_OG5SLVl9_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_PvUJpD8B_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_o00Cw8ZM_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_5acCGgWD_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_1yK8J2Xi_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_vhHphpFb_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_Uf12MfzV_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_BZHxabY7_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_nW5Y5uuG_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_H2rL1sKe_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_YkS0jmqa_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_rL2626FB_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_nCpPmiEw_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_Gf9bTgbH_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_URRZG7c1_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cSend_ddRf5nLe_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_524Ttv0r_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_TuOXAjLL_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_sLVsFFYY_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_xk53R03m_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_XuC6LsO4_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_etJ94GQB_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_FSSyGdQu_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_1FkrRRpj_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_s8eMEhs6_sendMessage(HeavyContextInterface *, int, const HvMessage *);

  // objects
  SignalLine sLine_4mtXeDd3;
  SignalEnvelope sEnv_qdUVNvVK;
  SignalEnvelope sEnv_n1iQiX09;
  SignalEnvelope sEnv_3yBGsFvL;
  SignalEnvelope sEnv_4j0v9jVS;
  SignalLine sLine_Py16NFVW;
  SignalLine sLine_SXKkNsV4;
  ControlBinop cBinop_Blx9xkxy;
  ControlBinop cBinop_CaHXMadz;
  ControlBinop cBinop_ndu9I1xW;
  ControlBinop cBinop_J0OzpbB4;
  ControlBinop cBinop_zJbEUjKd;
  ControlBinop cBinop_JPH2yCq9;
  ControlBinop cBinop_A8b4Ju0G;
  ControlBinop cBinop_a3RVo6aL;
  ControlVar cVar_OXe6zkEU;
  ControlVar cVar_A7H1C2I0;
  SignalVarf sVarf_S6dVnCsj;
  ControlVar cVar_4BT0EhSw;
  ControlIf cIf_GUZxd4I8;
  ControlIf cIf_xBntjQ0N;
  ControlBinop cBinop_rjX2mdES;
  ControlBinop cBinop_zC2pwnjX;
  ControlBinop cBinop_YMuzQT3t;
  ControlBinop cBinop_ryDYU6oF;
  ControlBinop cBinop_EtPBzWWD;
  SignalVarf sVarf_zCdL60Ya;
  ControlVar cVar_5e2sKyDz;
  ControlIf cIf_SC0qUaaC;
  ControlIf cIf_jCSbWVIL;
  ControlBinop cBinop_4gE23Gni;
  ControlBinop cBinop_PvUJpD8B;
  ControlBinop cBinop_o00Cw8ZM;
  ControlBinop cBinop_vhHphpFb;
  ControlBinop cBinop_BZHxabY7;
  ControlVar cVar_rL2626FB;
  ControlBinop cBinop_nCpPmiEw;
  SignalVarf sVarf_tYDAqoPn;
  SignalVarf sVarf_FWGGvTg0;
  SignalVarf sVarf_kl16ValL;
  SignalVarf sVarf_Go5uBuiR;
  ControlBinop cBinop_524Ttv0r;
};

#endif // _HEAVY_CONTEXT_WSTD_FLDR_HPP_
